@@ -420,14 +420,16 @@ defaults_refresh_screen_dimension_properties (Defaults *self)
 	gulong       items_left;
 	glong*       coords;
 	Atom         workarea_atom;
+	Display*     display;
 
 	g_return_if_fail ((self != NULL) && IS_DEFAULTS (self));
 
 	/* get real desktop-area without the panels */
 	workarea_atom = gdk_x11_get_xatom_by_name ("_NET_WORKAREA");
+	display = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
   
 	gdk_error_trap_push ();
-	result = XGetWindowProperty (GDK_DISPLAY (),
+	result = XGetWindowProperty (display,
 				     GDK_ROOT_WINDOW (),
 				     workarea_atom,
 				     0L,
@@ -2365,7 +2367,7 @@ _window_look_for_top_panel_attributes (GdkWindow *win)
 
 	gdk_error_trap_push ();
 
-	result = XGetClassHint (GDK_DISPLAY (),
+	result = XGetClassHint (gdk_x11_display_get_xdisplay (gdk_display_get_default ()),
 				GDK_WINDOW_XWINDOW (win),
 				&class_hints);
 
@@ -2440,6 +2442,7 @@ defaults_get_top_corner (Defaults *self, gint *x, gint *y)
 	gint         panel_monitor    = 0;
 	gint         aw_monitor;
 	gboolean     has_panel_window = FALSE;
+	gboolean     follow_focus     = defaults_multihead_does_focus_follow (self);
 
 	g_return_if_fail (self != NULL && IS_DEFAULTS (self));
 
@@ -2467,7 +2470,7 @@ defaults_get_top_corner (Defaults *self, gint *x, gint *y)
 		has_panel_window  = TRUE;
 	}
 
-	if (defaults_multihead_does_focus_follow (self))
+	if (follow_focus)
 	{
 		g_debug ("multi_head_focus_follow mode");
 		monitor = gdk_screen_get_monitor_at_point (screen, mx, my);
@@ -2505,7 +2508,7 @@ defaults_get_top_corner (Defaults *self, gint *x, gint *y)
 	{
 		/* position the corner on the selected monitor */
 		rect.y += panel_rect.y + panel_rect.height;
-	} else if (! has_panel_window)
+	} else if (! (has_panel_window || follow_focus))
 	{
 		g_debug ("no panel detetected; using workarea fallback");
 
