@@ -14,6 +14,7 @@
 **
 ** Contributor(s):
 **    Abhishek Mukherjee <abhishek.mukher.g@gmail.com> (append fixes, rev. 280)
+**    Aurélien Gâteau <aurelien.gateau@canonical.com> (0.10 spec, rev. 348)
 **
 ** This program is free software: you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License version 3, as published
@@ -36,11 +37,17 @@
 #include "stack.h"
 #include "bubble.h"
 #include "apport.h"
+#include "dialog.h"
 #include "dnd.h"
 #include "log.h"
 
 G_DEFINE_TYPE (Stack, stack, G_TYPE_OBJECT);
 
+#define FORCED_SHUTDOWN_THRESHOLD 500
+
+/* fwd declaration */
+void close_handler (GObject* n, Stack*  stack);
+
 /*-- internal API ------------------------------------------------------------*/
 
 static void
@@ -52,9 +59,12 @@ stack_dispose (GObject* gobject)
 
 static
 void
-delete_entry (gpointer data,
+disconnect_bubble (gpointer data,
 	      gpointer user_data)
 {
+	Bubble* bubble = BUBBLE(data);
+	Stack* stack = STACK(user_data);
+	g_signal_handlers_disconnect_by_func (G_OBJECT(bubble), G_CALLBACK (close_handler), stack);
 }
 
 
@@ -62,7 +72,7 @@ static void
 stack_finalize (GObject* gobject)
 {
 	if (STACK(gobject)->list != NULL)
-		g_list_foreach (STACK(gobject)->list, delete_entry, NULL);
+		g_list_foreach (STACK(gobject)->list, disconnect_bubble, gobject);
 	if (STACK(gobject)->defaults != NULL)
 		g_object_unref (STACK(gobject)->defaults);
 	if (STACK(gobject)->observer != NULL)
@@ -237,34 +247,13 @@ find_bubble_for_append(Stack* self,
 	return (Bubble*) entry->data;
 }
 
-
 static void
-stack_purge_old_bubbles (Stack* self)
+_weak_notify_cb (gpointer data,
+		 GObject* former_object)
 {
-	Bubble* bubble = NULL;
-	GList*    list = NULL;
+	Stack* stack = STACK (data);
 
-	g_return_if_fail (self != NULL);
-
-	for (list = g_list_first (self->list);
-	     list != NULL;)
-	{
-		bubble = (Bubble*) list->data;
-		
-		if (! IS_BUBBLE (bubble))
-		{
-			self->list = g_list_delete_link (self->list, list);
-			list = self->list;
-		} else if (! bubble_is_visible (bubble) &&
-			   (bubble_get_timeout (bubble) == 0))
-		{
-			self->list = g_list_delete_link (self->list, list);
-			list = self->list;
-			g_object_unref (bubble);
-		} else {
-			list = g_list_next (list);
-		}
-	}
+	stack->list = g_list_remove (stack->list, former_object);
 }
 
 static void
@@ -292,12 +281,6 @@ value_changed_handler (Defaults* defaults,
 		g_list_foreach (stack->list, _trigger_bubble_redraw, NULL);
 }
 
-/* fwd declaration */
-void close_handler (GObject* n, Stack*  stack);
-
-/* this is in dialog.c */
-gboolean dialog_check_actions_and_timeout (gchar **actions, gint timeout);
-
 static Bubble *sync_bubble = NULL;
 
 #include "display.c"
@@ -317,10 +300,12 @@ stack_new (Defaults* defaults,
 	if (!this)
 		return NULL;
 
-	this->defaults = defaults;
-	this->observer = observer;
-	this->list     = NULL;
-	this->next_id  = 1;
+	this->defaults           = defaults;
+	this->observer           = observer;
+	this->list               = NULL;
+	this->next_id            = 1;
+	this->slots[SLOT_TOP]    = NULL;
+	this->slots[SLOT_BOTTOM] = NULL;
 
 	/* hook up handler to act on changes of defaults/settings */
 	g_signal_connect (G_OBJECT (defaults),
@@ -352,11 +337,25 @@ close_handler (GObject *n,
 
 	if (n != NULL)
 	{
+		if (IS_BUBBLE (n))
+			stack_free_slot (stack, BUBBLE (n));
+
 		if (IS_BUBBLE (n)
 		    && bubble_is_synchronous (BUBBLE (n)))
 		{
 			g_object_unref (n);
 			sync_bubble = NULL;
+		} if (IS_BUBBLE (n)) {
+			stack_pop_bubble_by_id (stack, bubble_get_id ((Bubble*) n));
+			/* Fix for a tricky race condition
+			   where a bubble fades out in sync
+			   with a synchronous bubble: the symc.
+			   one is still considered visible while
+			   the normal one has triggered this signal.
+			   This ensures the display slot of the
+			   sync. bubble is recycled, and no gap is
+			   left on the screen */
+			sync_bubble = NULL;
 		} else {
 			/* Fix for a tricky race condition
 			   where a bubble fades out in sync
@@ -390,7 +389,7 @@ stack_push_bubble (Stack*  self,
 	/* check if this is just an update */
 	if (find_bubble_by_id (self, bubble_get_id (bubble)))
 	{
-		bubble_start_timer (bubble);
+		bubble_start_timer (bubble, TRUE);
 		bubble_refresh (bubble);
 
 		/* resync the synchronous bubble if it's at the top */
@@ -404,6 +403,11 @@ stack_push_bubble (Stack*  self,
 
 	/* add bubble/id to stack */
 	notification_id = self->next_id++;
+
+	// FIXME: migrate stack to use abstract notification object and don't
+	// keep heavy bubble objects around, at anyone time at max. only two
+	// bubble-objects will be in memory... this will also reduce leak-
+	// potential
 	bubble_set_id (bubble, notification_id);
 	self->list = g_list_append (self->list, (gpointer) bubble);
 
@@ -493,6 +497,69 @@ process_dbus_icon_data (GValue *data)
 	return pixbuf;
 }
 
+// control if there are non-default actions requested with this notification
+static gboolean
+dialog_check_actions_and_timeout (gchar** actions,
+				  gint    timeout)
+{
+	int      i                = 0;
+	gboolean turn_into_dialog = FALSE;
+
+	if (actions != NULL)
+	{
+		for (i = 0; actions[i] != NULL; i += 2)
+		{
+			if (actions[i+1] == NULL)
+			{
+				g_debug ("incorrect action callback "
+					 "with no label");
+				break;
+			}
+
+			turn_into_dialog = TRUE;        
+			g_debug ("notification request turned into a dialog "
+				 "box, because it contains at least one action "
+				 "callback (%s: \"%s\")",
+				 actions[i],
+				 actions[i+1]);
+		}
+
+		if (timeout == 0)
+		{
+			turn_into_dialog = TRUE;
+			g_debug ("notification request turned into a dialog "
+				 "box, because of its infinite timeout");
+		}
+	}
+
+	return turn_into_dialog;
+}
+
+// FIXME: a intnernal function used for the forcefully-shutdown work-around
+// regarding mem-leaks
+gboolean
+_arm_forced_quit (gpointer data)
+{
+	Stack* stack = NULL;
+
+	// sanity check, "disarming" this forced quit
+	if (!data)
+		return FALSE;
+
+	stack = STACK (data);
+
+	// only forcefully quit if the queue is empty
+	if (g_list_length (stack->list) == 0)
+	{
+		gtk_main_quit ();
+
+		// I don't think this is ever reached :)
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 gboolean
 stack_notify_handler (Stack*                 self,
 		      const gchar*           app_name,
@@ -507,18 +574,60 @@ stack_notify_handler (Stack*                 self,
 {
 	Bubble*    bubble     = NULL;
 	Bubble*    app_bubble = NULL;
-	GValue*      data     = NULL;
+	GValue*    data       = NULL;
 	GValue*    compat     = NULL;
 	GdkPixbuf* pixbuf     = NULL;
 	gboolean   new_bubble = FALSE;
+	gboolean   turn_into_dialog;
 
-        /* check if a bubble exists with same id */
+	// check max. allowed limit queue-size
+	if (g_list_length (self->list) > MAX_STACK_SIZE)
+	{
+		GError* error = NULL;
+
+		error = g_error_new (g_quark_from_string ("notify-osd"),
+		                     1,
+		                     "Reached stack-limit of %d",
+		                     MAX_STACK_SIZE);
+		dbus_g_method_return_error (context, error);
+		g_error_free (error);
+		error = NULL;
+
+		return TRUE;
+	}
+
+	// see if pathological actions or timeouts are used by an app issuing a
+	// notification
+	turn_into_dialog = dialog_check_actions_and_timeout (actions, timeout);
+	if (turn_into_dialog)
+	{
+		// TODO: apport_report (app_name, summary, actions, timeout);
+		gchar* sender = dbus_g_method_get_sender (context);
+
+		fallback_dialog_show (self->defaults,
+				      sender,
+				      app_name,
+				      id,
+				      summary,
+				      body,
+				      actions);
+		g_free (sender);
+		dbus_g_method_return (context, id);
+
+		return TRUE;
+	}
+
+        // check if a bubble exists with same id
 	bubble = find_bubble_by_id (self, id);
 	if (bubble == NULL)
 	{
 		gchar *sender;
 		new_bubble = TRUE;
 		bubble = bubble_new (self->defaults);
+		g_object_weak_ref (G_OBJECT (bubble),
+				   _weak_notify_cb,
+				   (gpointer) self);
+		
 		sender = dbus_g_method_get_sender (context);
 		bubble_set_sender (bubble, sender);
 		g_free (sender);
@@ -528,7 +637,9 @@ stack_notify_handler (Stack*                 self,
 	{
 		data   = (GValue*) g_hash_table_lookup (hints, "x-canonical-append");
 		compat = (GValue*) g_hash_table_lookup (hints, "append");
-		if (G_VALUE_HOLDS_STRING (data) || G_VALUE_HOLDS_STRING (compat))
+
+		if ((data && G_VALUE_HOLDS_STRING (data)) ||
+		    (compat && G_VALUE_HOLDS_STRING (compat)))
 			bubble_set_append (bubble, TRUE);
 		else
 			bubble_set_append (bubble, FALSE);
@@ -539,7 +650,6 @@ stack_notify_handler (Stack*                 self,
 	if (body)
 		bubble_set_message_body (bubble, body);
 
-
 	if (new_bubble && bubble_is_append_allowed(bubble)) {
 		app_bubble = find_bubble_for_append(self, bubble);
 
@@ -558,7 +668,7 @@ stack_notify_handler (Stack*                 self,
 	{
 		data   = (GValue*) g_hash_table_lookup (hints, "x-canonical-private-synchronous");
 		compat = (GValue*) g_hash_table_lookup (hints, "synchronous");
-		if (G_VALUE_HOLDS_STRING (data) || G_VALUE_HOLDS_STRING (compat))
+		if ((data && G_VALUE_HOLDS_STRING (data)) || (compat && G_VALUE_HOLDS_STRING (compat)))
 		{
 			if (sync_bubble != NULL
 			    && IS_BUBBLE (sync_bubble))
@@ -567,10 +677,10 @@ stack_notify_handler (Stack*                 self,
 				bubble = sync_bubble;
 			}
 
-			if (G_VALUE_HOLDS_STRING (data))
+			if (data && G_VALUE_HOLDS_STRING (data))
 				bubble_set_synchronous (bubble, g_value_get_string (data));
 
-			if (G_VALUE_HOLDS_STRING (compat))
+			if (compat && G_VALUE_HOLDS_STRING (compat))
 				bubble_set_synchronous (bubble, g_value_get_string (compat));
 		}
 	}
@@ -578,16 +688,16 @@ stack_notify_handler (Stack*                 self,
 	if (hints)
 	{
 		data = (GValue*) g_hash_table_lookup (hints, "value");
-		if (G_VALUE_HOLDS_INT (data))
+		if (data && G_VALUE_HOLDS_INT (data))
 			bubble_set_value (bubble, g_value_get_int (data));
 	}
 
 	if (hints)
 	{
 		data = (GValue*) g_hash_table_lookup (hints, "urgency");
-		if (G_VALUE_HOLDS_UCHAR (data))
-			bubble_set_urgent (bubble,
-					   (g_value_get_uchar (data) == 2));
+		if (data && G_VALUE_HOLDS_UCHAR (data))
+			bubble_set_urgency (bubble,
+					   g_value_get_uchar (data));
 		/* Note: urgency was defined as an enum: LOW, NORMAL, CRITICAL
 		   So, 2 means CRITICAL
 		*/
@@ -597,7 +707,7 @@ stack_notify_handler (Stack*                 self,
 	{
 		data   = (GValue*) g_hash_table_lookup (hints, "x-canonical-private-icon-only");
 		compat = (GValue*) g_hash_table_lookup (hints, "icon-only");
-		if (G_VALUE_HOLDS_STRING (data) || G_VALUE_HOLDS_STRING (compat))
+		if ((data && G_VALUE_HOLDS_STRING (data)) || (compat && G_VALUE_HOLDS_STRING (compat)))
 			bubble_set_icon_only (bubble, TRUE);
 		else
 			bubble_set_icon_only (bubble, FALSE);
@@ -605,38 +715,37 @@ stack_notify_handler (Stack*                 self,
 
 	if (hints)
 	{
-		data = (GValue*) g_hash_table_lookup (hints, "icon_data");
-		if (*icon == '\0' && data != NULL)
+		if ((data = (GValue*) g_hash_table_lookup (hints, "image_data")))
 		{
+			g_debug("Using image_data hint\n");
 			pixbuf = process_dbus_icon_data (data);
 			bubble_set_icon_from_pixbuf (bubble, pixbuf);
-		} else
+		}
+		else if ((data = (GValue*) g_hash_table_lookup (hints, "image_path")))
+		{
+			g_debug("Using image_path hint\n");
+			if ((data && G_VALUE_HOLDS_STRING (data)))
+				bubble_set_icon_from_path (bubble, g_value_get_string(data));
+			else
+				g_warning ("image_path hint is not a string\n");
+		}
+		else if (icon && *icon != '\0')
+		{
+			g_debug("Using icon parameter\n");
 			bubble_set_icon (bubble, icon);
+		}
+		else if ((data = (GValue*) g_hash_table_lookup (hints, "icon_data")))
+		{
+			g_debug("Using deprecated icon_data hint\n");
+			pixbuf = process_dbus_icon_data (data);
+			bubble_set_icon_from_pixbuf (bubble, pixbuf);
+		}
 	}
 
 	log_bubble_debug (bubble, app_name,
 			  (*icon == '\0' && data != NULL) ?
 			  "..." : icon);
 
-	gboolean turn_into_dialog = dialog_check_actions_and_timeout (actions, timeout);
-
-	if (turn_into_dialog)
-		/* || bubble_is_urgent (bubble)) */
-	{
-		/* TODO: apport_report (app_name, summary, actions, timeout); */
-
-		GObject *dialog = G_OBJECT (
-			bubble_show_dialog (bubble, app_name, actions));
-		g_signal_connect (dialog,
-				  "destroy-event",
-				  G_CALLBACK (close_handler),
-				  self);
-
-		dbus_g_method_return (context, bubble_get_id (bubble));
-
-		return TRUE;
-	}
-
 	bubble_determine_layout (bubble);
 
 	bubble_recalc_size (bubble);
@@ -644,7 +753,6 @@ stack_notify_handler (Stack*                 self,
 	if (bubble_is_synchronous (bubble))
 	{
 		stack_display_sync_bubble (self, bubble);
-
 	} else {
 		stack_push_bubble (self, bubble);
 
@@ -658,16 +766,30 @@ stack_notify_handler (Stack*                 self,
 		/* make sure the sync. bubble is positioned correctly
 		   even for the append case
 		*/
-		if (sync_bubble != NULL
-		    && bubble_is_visible (sync_bubble))
-			stack_display_position_sync_bubble (self, sync_bubble);
+		// no longer needed since we have the two-slots mechanism now
+		//if (sync_bubble != NULL
+		//    && bubble_is_visible (sync_bubble))
+		//	stack_display_position_sync_bubble (self, sync_bubble);
 
 		/* update the layout of the stack;
 		 * this will also open the new bubble */
 		stack_layout (self);
 	}
 
-	dbus_g_method_return (context, bubble_get_id (bubble));
+	if (bubble)
+		dbus_g_method_return (context, bubble_get_id (bubble));
+
+	// FIXME: this is a temporary work-around, I do not like at all, until
+	// the heavy memory leakage of notify-osd is fully fixed...
+	// after a threshold-value is reached, "arm" a forceful shutdown of
+	// notify-osd (still allowing notifications in the queue, and coming in,
+	// to be displayed), in order to get the leaked memory freed again, any
+	// new notifications, coming in after the shutdown, will instruct the
+	// session to restart notify-osd
+	if (bubble_get_id (bubble) == FORCED_SHUTDOWN_THRESHOLD)
+		g_timeout_add (defaults_get_on_screen_timeout (self->defaults),
+			       _arm_forced_quit,
+			       (gpointer) self);
 
 	return TRUE;
 }
@@ -677,20 +799,27 @@ stack_close_notification_handler (Stack*   self,
 				  guint    id,
 				  GError** error)
 {
-	Bubble *bubble = find_bubble_by_id (self, id);
+	if (id == 0)
+		g_warning ("%s(): notification id == 0, likely wrong\n",
+			   G_STRFUNC);
 
-	/* exit but pretend it's ok, for applications
-	   that call us after an action button was clicked */
+	Bubble* bubble = find_bubble_by_id (self, id);
+
+	// exit but pretend it's ok, for applications
+	// that call us after an action button was clicked
 	if (bubble == NULL)
 		return TRUE;
 
 	dbus_send_close_signal (bubble_get_sender (bubble),
 				bubble_get_id (bubble),
 				3);
-	bubble_hide (bubble);
-	g_object_unref (bubble);
 
-	stack_layout (self);
+	// do not trigger any closure of a notification-bubble here, as
+	// this kind of control from outside (DBus) does not comply with
+	// the notify-osd specification
+	//bubble_hide (bubble);
+	//g_object_unref (bubble);
+	//stack_layout (self);
 
 	return TRUE;
 }
@@ -733,7 +862,165 @@ stack_get_server_information (Stack*  self,
 	*out_name     = g_strdup ("notify-osd");
 	*out_vendor   = g_strdup ("Canonical Ltd");
 	*out_version  = g_strdup ("1.0");
-	*out_spec_ver = g_strdup ("0.9");
+	*out_spec_ver = g_strdup ("1.1");
 
 	return TRUE;
 }
+
+gboolean
+stack_is_slot_vacant (Stack* self,
+                      Slot   slot)
+{
+	// sanity checks
+	if (!self || !IS_STACK (self))
+		return FALSE;
+
+	if (slot != SLOT_TOP && slot != SLOT_BOTTOM)
+		return FALSE;
+
+	return self->slots[slot] == NULL ? VACANT : OCCUPIED;
+}
+
+// return values of -1 for x and y indicate an error by the caller
+void
+stack_get_slot_position (Stack* self,
+                         Slot   slot,
+                         gint   bubble_height,
+                         gint*  x,
+                         gint*  y)
+{
+	// sanity checks
+	if (!x && !y)
+		return;
+
+	if (!self || !IS_STACK (self))
+	{
+		*x = -1;
+		*y = -1;
+		return;
+	}
+
+	if (slot != SLOT_TOP && slot != SLOT_BOTTOM)
+	{
+		*x = -1;
+		*y = -1;
+		return;
+	}
+
+	// initialize x and y
+	defaults_get_top_corner (self->defaults, x, y);
+
+	// differentiate returned top-left corner for top and bottom slot
+	// depending on the placement 
+	switch (defaults_get_gravity (self->defaults))
+	{
+		Defaults* d;
+
+		case GRAVITY_EAST:
+			d = self->defaults;
+
+			// the position for the sync./feedback bubble
+			if (slot == SLOT_TOP)
+				*y += defaults_get_desktop_height (d) / 2 -
+				      EM2PIXELS (defaults_get_bubble_vert_gap (d) / 2.0f, d) -
+				      bubble_height +
+				      EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
+			// the position for the async. bubble
+			else if (slot == SLOT_BOTTOM)
+				*y += defaults_get_desktop_height (d) / 2 +
+				      EM2PIXELS (defaults_get_bubble_vert_gap (d) / 2.0f, d) -
+				      EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
+		break;
+
+		case GRAVITY_NORTH_EAST:
+			d = self->defaults;
+
+			// there's nothing to do for slot == SLOT_TOP as we
+			// already have correct x and y from the call to
+			// defaults_get_top_corner() earlier
+
+			// this needs to look at the height of the bubble in the
+			// top slot
+			if (slot == SLOT_BOTTOM)
+			{
+				switch (defaults_get_slot_allocation (d))
+				{
+					case SLOT_ALLOCATION_FIXED:
+						*y += EM2PIXELS (defaults_get_icon_size (d), d) +
+						      2 * EM2PIXELS (defaults_get_margin_size (d), d) +
+						      EM2PIXELS (defaults_get_bubble_vert_gap (d), d); /* +
+						      2 * EM2PIXELS (defaults_get_bubble_shadow_size (d), d);*/
+					break;
+
+					case SLOT_ALLOCATION_DYNAMIC:
+						g_assert (stack_is_slot_vacant (self, SLOT_TOP) == OCCUPIED);
+						*y += bubble_get_height (self->slots[SLOT_TOP]) +
+						      EM2PIXELS (defaults_get_bubble_vert_gap (d), d) -
+						      2 * EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
+					break;
+
+					default:
+					break;
+				}
+
+			}
+		break;
+
+		default:
+			g_warning ("Unhandled placement!\n");
+		break;
+	}
+}
+
+// call this _before_ the fade-in animation of the bubble starts
+gboolean
+stack_allocate_slot (Stack*  self,
+		     Bubble* bubble,
+                     Slot    slot)
+{
+	// sanity checks
+	if (!self || !IS_STACK (self))
+		return FALSE;
+
+	if (!bubble || !IS_BUBBLE (bubble))
+		return FALSE;
+
+	if (slot != SLOT_TOP && slot != SLOT_BOTTOM)
+		return FALSE;
+
+	if (stack_is_slot_vacant (self, slot))
+		self->slots[slot] = BUBBLE (g_object_ref ((gpointer) bubble));
+	else
+		return FALSE;
+
+	return TRUE;
+}
+
+// call this _after_ the fade-out animation of the bubble is finished
+gboolean
+stack_free_slot (Stack*  self,
+		 Bubble* bubble)
+{
+	// sanity checks
+	if (!self || !IS_STACK (self))
+		return FALSE;
+
+	if (!bubble || !IS_BUBBLE (bubble))
+		return FALSE;
+
+	// check top and bottom slots for bubble pointer equality
+	if (bubble == self->slots[SLOT_TOP])
+	{
+		g_object_unref (self->slots[SLOT_TOP]);
+		self->slots[SLOT_TOP] = NULL;
+	}
+	else if (bubble == self->slots[SLOT_BOTTOM])
+	{
+		g_object_unref (self->slots[SLOT_BOTTOM]);
+		self->slots[SLOT_BOTTOM] = NULL;
+	}
+	else
+		return FALSE;
+
+	return TRUE;	
+}
